#ifndef __MAKET_TCP_H__
#define __MAKET_TCP_H__

#include <list>
#include <functional>
#include "../util/config.h"
#include "InstanceManager.h"

class EthernetMaketClient : public EthernetClient, public InstanceManager<EthernetMaketClient> {
protected:
    uint32_t lastActivityTime = 0; // Время последней активности
    const uint16_t timeout = 300;  // Таймаут неактивности в миллисекундах
    uint32_t startConnectionTime = 0;
    const uint16_t connectionTimeout = 350;
    uint32_t stopStartTime = 0;       // Время начала состояния CONNECT_IDLE
    const uint32_t stopTimeout = 300; // Таймаут для закрытия сокета в состоянии CONNECT_IDLE

    std::function<void()> onSendSuccess = []() {};

    bool isNonBlocking = true; // Флаг для определения режима подключения

    uint8_t* dataPtr = nullptr;
    uint16_t dataSize = 0;
    IPAddress dstIP;
    uint16_t dstPort;

    bool dataWrite();

    // static std::list<EthernetMaketClient*>& getInstances(); // Ленивое создание списка

    ConnectionStatusSimple connectStatus = CONNECT_IDLE; 

public:
    EthernetMaketClient();
    EthernetMaketClient(uint8_t sock);
    EthernetMaketClient(EthernetClient &client);
    
    //     // Move constructor
    // EthernetMaketClient(EthernetMaketClient&& other) noexcept
    //     : EthernetClient(std::move(other)), // Move base class
    //       lastActivityTime(other.lastActivityTime),
    //       startConnectionTime(other.startConnectionTime),
    //       stopStartTime(other.stopStartTime),
    //       onSendSuccess(std::move(other.onSendSuccess)),
    //       isNonBlocking(other.isNonBlocking),
    //       dataPtr(other.dataPtr),
    //       dataSize(other.dataSize),
    //       dstIP(std::move(other.dstIP)),
    //       dstPort(other.dstPort),
    //       connectStatus(other.connectStatus)
    // {
    //     other.dataPtr = nullptr; // Reset the source object's pointer
    //     other.dataSize = 0;
    // }

    EthernetMaketClient& operator=(const EthernetMaketClient& other);

    ConnectionStatusSimple connectNonBlock(IPAddress ip, uint16_t port);
    void startConnection(IPAddress ip, uint16_t port, bool nonBlock = true);

    void dataWrite(uint8_t* data, uint16_t dataSize, bool override = false);
    void clearData();
    bool isDataEmpty() const;

    void disconnect();
    void close();
    void stop();
    bool canWriteIntoConnection(size_t = 0) const;

    static void tick();
    void tick_();

    void setOnSendSuccess(const std::function<void()>& callback);
    void resetOnSendSuccess();

    ConnectionStatusSimple getConnectionStatus() const;

    void activityUpdate();
};

#endif // __MAKET_TCP_H__
